@@ -1,30 +1,8 @@
 /*
- * Copyright (c) 2005-2007 Douglas Gilbert.
+ * Copyright (c) 2005-2014 Douglas Gilbert.
  * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
+ * Use of this source code is governed by a BSD-style
+ * license that can be found in the BSD_LICENSE file.
  */
 
 #include <unistd.h>
@@ -45,18 +23,16 @@
 #include "sg_cmds_basic.h"
 #include "sg_cmds_extra.h"
 
-/* A utility program for the Linux OS SCSI subsystem.
+/* A utility program originally written for the Linux OS SCSI subsystem.
  *
  * This utility invokes the REASSIGN BLOCKS SCSI command to reassign
  * an existing (possibly damaged) lba on a direct access device (e.g.
- * a disk) to a new physical location. The previous contents is 
+ * a disk) to a new physical location. The previous contents is
  * recoverable then it is written to the remapped lba otherwise
  * vendor specific data is written.
  */
 
-static char * version_str = "1.11 20070925";
-
-#define ME "sg_reassign: "
+static const char * version_str = "1.15 20140126";
 
 #define DEF_DEFECT_LIST_FORMAT 4        /* bytes from index */
 
@@ -108,7 +84,7 @@ usage()
           "    --longlist=0|1\n"
           "       -l 0|1           use 4 byte list length when 1, safe to "
           "ignore\n"
-          "                        which default to 2 byte list length\n"
+          "                        (def: 0 (2 byte list length))\n"
           "    --primary|-p        fetch primary defect list length, "
           "don't reassign\n"
           "    --verbose|-v        increase verbosity\n"
@@ -117,27 +93,24 @@ usage()
           );
 }
 
-/* Trying to decode multipliers as sg_get_llnum() [in sg_libs does] would
+/* Trying to decode multipliers as sg_get_llnum() [in sg_libs] does would
  * only confuse things here, so use this local trimmed version */
-long long
+static int64_t
 get_llnum(const char * buf)
 {
     int res, len;
-    long long num;
-    unsigned long long unum;
-    const char * commap;
+    int64_t num;
+    uint64_t unum;
 
     if ((NULL == buf) || ('\0' == buf[0]))
         return -1LL;
-    len = strlen(buf);
-    commap = strchr(buf + 1, ',');
+    len = strspn(buf, "0123456789aAbBcCdDeEfFhHxX");
+    if (0 == len)
+        return -1LL;
     if (('0' == buf[0]) && (('x' == buf[1]) || ('X' == buf[1]))) {
         res = sscanf(buf + 2, "%" SCNx64 "", &unum);
         num = unum;
-    } else if (commap && ('H' == toupper(*(commap - 1)))) {
-        res = sscanf(buf, "%" SCNx64 "", &unum);
-        num = unum;
-    } else if ((NULL == commap) && ('H' == toupper(buf[len - 1]))) {
+    } else if ('H' == toupper(buf[len - 1])) {
         res = sscanf(buf, "%" SCNx64 "", &unum);
         num = unum;
     } else
@@ -148,19 +121,20 @@ get_llnum(const char * buf)
         return -1LL;
 }
 
-/* Read numbers (up to 64 bits in size) from command line (comma separated */
-/* list) or from stdin (one per line, comma separated list or */
-/* space separated list). Assumed decimal unless prefixed by '0x', '0X' */
-/* or contains trailing 'h' or 'H' (which indicate hex). */
-/* Returns 0 if ok, or 1 if error. */
+/* Read numbers (up to 64 bits in size) from command line (comma (or
+ * (single) space) separated list) or from stdin (one per line, comma
+ * separated list or space separated list). Assumed decimal unless prefixed
+ * by '0x', '0X' or contains trailing 'h' or 'H' (which indicate hex).
+ * Returns 0 if ok, or 1 if error. */
 static int
-build_lba_arr(const char * inp, unsigned long long * lba_arr,
+build_lba_arr(const char * inp, uint64_t * lba_arr,
               int * lba_arr_len, int max_arr_len)
 {
     int in_len, k, j, m;
     const char * lcp;
-    long long ll;
+    int64_t ll;
     char * cp;
+    char * c2p;
 
     if ((NULL == inp) || (NULL == lba_arr) ||
         (NULL == lba_arr_len))
@@ -170,12 +144,13 @@ build_lba_arr(const char * inp, unsigned long long * lba_arr,
     if (0 == in_len)
         *lba_arr_len = 0;
     if ('-' == inp[0]) {        /* read from stdin */
-        char line[512];
+        char line[1024];
         int off = 0;
 
         for (j = 0; j < 512; ++j) {
             if (NULL == fgets(line, sizeof(line), stdin))
                 break;
+            // could improve with carry_over logic if sizeof(line) too small
             in_len = strlen(line);
             if (in_len > 0) {
                 if ('\n' == line[in_len - 1]) {
@@ -183,7 +158,7 @@ build_lba_arr(const char * inp, unsigned long long * lba_arr,
                     line[in_len] = '\0';
                 }
             }
-            if (0 == in_len)
+            if (in_len < 1)
                 continue;
             lcp = line;
             m = strspn(lcp, " \t");
@@ -207,7 +182,7 @@ build_lba_arr(const char * inp, unsigned long long * lba_arr,
                                 "exceeded\n");
                         return 1;
                     }
-                    lba_arr[off + k] = (unsigned long long)ll;
+                    lba_arr[off + k] = (uint64_t)ll;
                     lcp = strpbrk(lcp, " ,\t");
                     if (NULL == lcp)
                         break;
@@ -229,7 +204,7 @@ build_lba_arr(const char * inp, unsigned long long * lba_arr,
         }
         *lba_arr_len = off;
     } else {        /* list of numbers (default decimal) on command line */
-        k = strspn(inp, "0123456789aAbBcCdDeEfFhHxX,");
+        k = strspn(inp, "0123456789aAbBcCdDeEfFhHxX, ");
         if (in_len != k) {
             fprintf(stderr, "build_lba_arr: error at pos %d\n", k + 1);
             return 1;
@@ -237,10 +212,15 @@ build_lba_arr(const char * inp, unsigned long long * lba_arr,
         for (k = 0; k < max_arr_len; ++k) {
             ll = get_llnum(lcp);
             if (-1 != ll) {
-                lba_arr[k] = (unsigned long long)ll;
-                cp = strchr(lcp, ',');
+                lba_arr[k] = (uint64_t)ll;
+                cp = (char *)strchr(lcp, ',');
+                c2p = (char *)strchr(lcp, ' ');
+                if (NULL == cp)
+                    cp = c2p;
                 if (NULL == cp)
                     break;
+                if (c2p && (c2p < cp))
+                    cp = c2p;
                 lcp = cp + 1;
             } else {
                 fprintf(stderr, "build_lba_arr: error at pos %d\n",
@@ -272,7 +252,7 @@ main(int argc, char * argv[])
     int primary = 0;
     int verbose = 0;
     const char * device_name = NULL;
-    unsigned long long addr_arr[MAX_NUM_ADDR];
+    uint64_t addr_arr[MAX_NUM_ADDR];
     unsigned char param_arr[4 + (MAX_NUM_ADDR * 8)];
     int param_len = 4;
     int ret = 0;
@@ -333,7 +313,7 @@ main(int argc, char * argv[])
             ++verbose;
             break;
         case 'V':
-            fprintf(stderr, ME "version: %s\n", version_str);
+            fprintf(stderr, "version: %s\n", version_str);
             return 0;
         default:
             fprintf(stderr, "unrecognised option code 0x%x ??\n", c);
@@ -412,17 +392,17 @@ main(int argc, char * argv[])
 
     sg_fd = sg_cmds_open_device(device_name, 0 /* rw */, verbose);
     if (sg_fd < 0) {
-        fprintf(stderr, ME "open error: %s: %s\n", device_name,
+        fprintf(stderr, "open error: %s: %s\n", device_name,
                 safe_strerror(-sg_fd));
         return SG_LIB_FILE_ERROR;
-    } 
+    }
 
     if (got_addr) {
         if (dummy) {
             fprintf(stderr, ">>> dummy: REASSIGN BLOCKS not executed\n");
             if (verbose) {
                 fprintf(stderr, "  Would have reassigned these blocks:\n");
-                for (j = 0; j < addr_arr_len; ++j) 
+                for (j = 0; j < addr_arr_len; ++j)
                     printf("    0x%" PRIx64 "\n", addr_arr[j]);
             }
             return 0;
