@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2010 Douglas Gilbert.
+ * Copyright (c) 2004-2014 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -11,6 +11,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <getopt.h>
+#include <ctype.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -24,7 +25,7 @@
  * mode page on the given device.
  */
 
-static char * version_str = "1.10 20100312";
+static const char * version_str = "1.12 20140110";
 
 #define ME "sg_wr_mode: "
 
@@ -111,8 +112,11 @@ static int build_mode_page(const char * inp, unsigned char * mp_arr,
         *mp_arr_len = 0;
     if ('-' == inp[0]) {        /* read from stdin */
         char line[512];
+        char carry_over[4];
         int off = 0;
+        int split_line;
 
+        carry_over[0] = 0;
         for (j = 0; j < 512; ++j) {
             if (NULL == fgets(line, sizeof(line), stdin))
                 break;
@@ -121,11 +125,32 @@ static int build_mode_page(const char * inp, unsigned char * mp_arr,
                 if ('\n' == line[in_len - 1]) {
                     --in_len;
                     line[in_len] = '\0';
-                }
+                    split_line = 0;
+                } else
+                    split_line = 1;
             }
-            if (0 == in_len)
+            if (in_len < 1) {
+                carry_over[0] = 0;
                 continue;
-            lcp = line;
+            }
+            if (carry_over[0]) {
+                if (isxdigit(line[0])) {
+                    carry_over[1] = line[0];
+                    carry_over[2] = '\0';
+                    if (1 == sscanf(carry_over, "%x", &h))
+                        mp_arr[off - 1] = h;       /* back up and overwrite */
+                    else {
+                        fprintf(stderr, "build_mode_page: carry_over error "
+                                "['%s'] around line %d\n", carry_over, j + 1);
+                        return 1;
+                    }
+                    lcp = line + 1;
+                    --in_len;
+                } else
+                    lcp = line;
+                carry_over[0] = 0;
+            } else
+                lcp = line;
             m = strspn(lcp, " \t");
             if (m == in_len)
                 continue;
@@ -147,6 +172,10 @@ static int build_mode_page(const char * inp, unsigned char * mp_arr,
                                 j + 1, (int)(lcp - line + 1));
                         return 1;
                     }
+                    if (split_line && (1 == strlen(lcp))) {
+                        /* single trailing hex digit might be a split pair */
+                        carry_over[0] = *lcp;
+                    }
                     if ((off + k) >= max_arr_len) {
                         fprintf(stderr, "build_mode_page: array length "
                                 "exceeded\n");
@@ -187,8 +216,8 @@ static int build_mode_page(const char * inp, unsigned char * mp_arr,
                     return 1;
                 }
                 mp_arr[k] = h;
-                cp = strchr(lcp, ',');
-                c2p = strchr(lcp, ' ');
+                cp = (char *)strchr(lcp, ',');
+                c2p = (char *)strchr(lcp, ' ');
                 if (NULL == cp)
                     cp = c2p;
                 if (NULL == cp)
@@ -247,8 +276,8 @@ static int build_mask(const char * inp, unsigned char * mask_arr,
                     return 1;
                 }
                 mask_arr[k] = h;
-                cp = strchr(lcp, ',');
-                c2p = strchr(lcp, ' ');
+                cp = (char *)strchr(lcp, ',');
+                c2p = (char *)strchr(lcp, ' ');
                 if (NULL == cp)
                     cp = c2p;
                 if (NULL == cp)
@@ -497,7 +526,7 @@ int main(int argc, char * argv[])
         if (! force) {
             if ((! (ref_md[off] & 0x80)) && save) {
                 fprintf(stderr, "PS bit in existing mode page indicates that "
-                        "it is not savable\n    but '--save' option given\n");
+                        "it is not saveable\n    but '--save' option given\n");
                 goto err_out;
             }
             read_in[0] &= 0x7f; /* mask out PS bit, reserved in mode select */
